@@ -32,10 +32,10 @@
           foreign-variable foreign-function))
 
 (eval-when (load compile eval)
-  (import (intern "*COUTPUT-FILE*" "COMPILER"))
-  (import (intern "*COUTPUT-STREAM*" "COMPILER"))
-  (import (intern "*FFI-MODULE*" "COMPILER"))
-  (import (intern "FINALIZE-COUTPUT-FILE" "COMPILER"))
+  (import (intern "*COUTPUT-FILE*" "SYSTEM"))
+  (import (intern "*COUTPUT-STREAM*" "SYSTEM"))
+  (import (intern "*FFI-MODULE*" "SYSTEM"))
+  (import (intern "FINALIZE-COUTPUT-FILE" "SYSTEM"))
   (import (intern "TEXT" "SYSTEM")) ; messages
   (import (intern "SYMBOL-TO-KEYWORD" "SYSTEM"))
   (import (intern "CHECK-SYMBOL" "SYSTEM")) ; error checking
@@ -213,8 +213,12 @@
                  typespec))
         (when name (setf (gethash name *c-type-table*) c-type))
         c-type)
-      (error (TEXT "FFI type should be a symbol, not ~S")
-             typespec))
+      (if (stringp typespec)
+        (let ((c-type (parse-foreign-inttype typespec)))
+          (when name (setf (gethash name *c-type-table*) c-type))
+          c-type)
+        (error (TEXT "FFI type should be a symbol, not ~S")
+               typespec)))
     (flet ((invalid (typespec)
              (error (TEXT "Invalid FFI type: ~S")
                     typespec))
@@ -453,7 +457,7 @@
                (list slot (deparse slottype)))
              (deparse (ctype)
                (or (cdr (assoc ctype alist :test #'eq))
-                   (if (symbolp ctype)
+                   (if (or (symbolp ctype) (stringp ctype))
                      ;; <simple-c-type>, c-pointer, c-string
                      (new-type ctype ctype)
                      (let ((typespec (list (svref ctype 0))))
@@ -499,7 +503,7 @@
                                                 ((null args) (nreverse argspecs))
                                               (let ((argtype (first args))
                                                     (argflags (second args)))
-                                                (push `(,(intern (format nil "arg~D" i) compiler::*keyword-package*)
+                                                (push `(,(intern (format nil "arg~D" i) system::*keyword-package*)
                                                         ,(deparse argtype)
                                                         ,(cond ((flag-set-p argflags ff-flag-out) ':OUT)
                                                                ((flag-set-p argflags ff-flag-in-out) ':IN-OUT)
@@ -834,7 +838,7 @@
 ; Pass an object from the compilation environment to the module.
 (defun pass-object (object)
   (new-object t
-              (let ((*package* compiler::*keyword-package*))
+              (let ((*package* system::*keyword-package*))
                 (write-to-string object :readably t :pretty nil))))
 
 ; Convert an object's index to a C lvalue.
@@ -846,7 +850,7 @@
   `(EVAL-WHEN (COMPILE)
      (DO-C-LINES ,format-string ,@args)))
 (defun do-c-lines (format-string &rest args) ; ABI
-  (when (compiler::prepare-coutput-file)
+  (when (system::prepare-coutput-file)
     (prepare-module)
     (etypecase format-string
       ((or string function)
@@ -869,6 +873,12 @@
   (let ((pair (assoc key alist)))
     (if pair (second pair) default)))
 
+(defun maximize-integer-type (type)
+  (case type
+    ((char short int long sint8 sint16 sint32 sint64) 'long)
+    ((uchar ushort uint ulong uint8 uint16 uint32 uint64) 'ulong)
+    (t type)))
+
 ;; CPP consts (#define'd in an *.h file):
 ;; for each type (int, string, pointer) there is a C function (and a
 ;;  foreign-function created when the first constant of this type is
@@ -880,21 +890,18 @@
   (let* ((alist (parse-options options '(:name :type :documentation :guard)
                                whole-form))
          (doc (cdr (assoc ':documentation alist))) ; ("doc string") or NIL
-         (c-type (get-assoc :type alist 'ffi:int))
+         (c-type (maximize-integer-type (get-assoc :type alist 'ffi:long)))
          (c-name (get-assoc :name alist name))
          (guard (get-assoc :guard alist (format nil "defined(~A)" c-name)))
          (cftype (parse-c-function
                   `((:arguments (number int) (defined-p (c-ptr int) :out))
                     (:return-type ,c-type))
                   whole-form)))
-    (check-type c-type (member ffi:int ffi:c-string ffi:c-pointer)
+    (check-type c-type (member ffi:long ffi:ulong ffi:c-string ffi:c-pointer)
                 "an integer, a string or a pointer")
     `(let ((f-name&c-number
             (compile-time-value (note-c-const ',c-name ',c-type
                                               ',cftype ',guard))))
-       (unless f-name&c-number
-         (error (TEXT "~S(~S ~S) requires writing to a C file")
-                'def-c-const ',name ',c-name))
        (let ((f-name (first f-name&c-number))
              (c-number (second f-name&c-number)))
          ;; c-number == 0 ==> need to output the def-call-out form
@@ -907,26 +914,27 @@
          (defconstant ,name (c-const-value f-name c-number ',name ',c-name)
            ,@doc)))))
 
-(defun note-c-const (c-name c-type cftype guard)
-  (when (compiler::prepare-coutput-file)
-    (prepare-module)
-    (let ((f-name (intern
-                   (format nil "module__~A__constant_map_~A" *name*
-                           (nstring-downcase
-                            (nsubstitute #\_ #\-
-                                         (copy-seq (symbol-name c-type))))))))
-      (list f-name
-            (vector-push-extend
-             (cons c-name guard)
-             (second (or (gethash c-type *constant-table*)
-                         (setf (gethash c-type *constant-table*)
-                               (list f-name (make-array 10 :adjustable t
-                                                        :fill-pointer 0)
-                                     cftype)))))))))
+(defun note-c-const (c-name c-type cftype guard) ; ABI
+  (unless (system::prepare-coutput-file)
+    (error (TEXT "~S(~S) requires writing to a C file") 'def-c-const c-name))
+  (prepare-module)
+  (let ((f-name (intern
+                 (format nil "module__~A__constant_map_~A" *name*
+                         (nstring-downcase
+                          (nsubstitute #\_ #\-
+                                       (copy-seq (symbol-name c-type))))))))
+    (list f-name
+          (vector-push-extend
+           (cons c-name guard)
+           (second (or (gethash c-type *constant-table*)
+                       (setf (gethash c-type *constant-table*)
+                             (list f-name (make-array 10 :adjustable t
+                                                      :fill-pointer 0)
+                                   cftype))))))))
 
 (defun c-const-value (f-name c-number name c-name) ; ABI
   (multiple-value-bind (value value-p) (funcall f-name c-number)
-    (if value-p value
+    (if (plusp value-p) value
         (progn
           (warn (TEXT "~S(~S): CPP constant ~A is not defined")
                 'def-c-const name c-name)
@@ -984,7 +992,7 @@
        ',name)))
 
 (defun note-c-var (c-name type flags) ; ABI
-  (when (compiler::prepare-coutput-file)
+  (when (system::prepare-coutput-file)
     (prepare-module)
     (push (list c-name (parse-c-type type) flags) *variable-list*)))
 
@@ -1078,7 +1086,7 @@
                                    :built-in :library :version :documentation)
                          whole-form))
          (def (gensym "DEF-CALL-OUT-"))
-         (properties (and (>= 1 (compiler::declared-optimize
+         (properties (and (>= 1 (system::declared-optimize
                                  'space (and (boundp 'system::*denv*)
                                              system::*denv*)))
                           (assoc ':documentation alist)))
@@ -1094,15 +1102,15 @@
        (EXT:COMPILER-LET ((,def ,ctype))
          (EVAL-WHEN (COMPILE)
            (UNLESS ,LIBRARY (NOTE-C-FUN ',c-name ,def ',built-in)))
-         (COMPILER::EVAL-WHEN-COMPILE
-           (COMPILER::C-DEFUN ',name (C-TYPE-TO-SIGNATURE ,ctype))))
+         (SYSTEM::EVAL-WHEN-COMPILE
+           (SYSTEM::C-DEFUN ',name (C-TYPE-TO-SIGNATURE ,ctype))))
        (WHEN ,def                       ; found library function
          (SYSTEM::REMOVE-OLD-DEFINITIONS ',name)
          (SYSTEM::%PUTD ',name ,def))
        ',name)))
 
 (defun note-c-fun (c-name ctype built-in) ; not ABI, compile-time only
-  (when (compiler::prepare-coutput-file)
+  (when (system::prepare-coutput-file)
     (prepare-module)
     (push (list c-name ctype built-in)
           *function-list*)))
@@ -1191,7 +1199,7 @@
                    "mallocing" "nomalloc")))))
 
 (defun note-c-call-in (name c-name alist whole) ; ABI
-  (when (compiler::prepare-coutput-file)
+  (when (system::prepare-coutput-file)
     (prepare-module)
     (let* ((fvd (parse-c-function alist whole))
            (rettype (svref fvd 1))
