@@ -1,6 +1,6 @@
 ;;; Inspect
 ;;;
-;;; Copyright (C) 2000 by Sam Steingold
+;;; Copyright (C) 2000-2008 by Sam Steingold
 ;;; This is Free Software, covered by the GNU GPL (v2)
 ;;; See http://www.gnu.org/copyleft/gpl.html
 ;;;
@@ -26,17 +26,21 @@
 ;;; utilities from cllib
 ;;;
 
+(in-package "CUSTOM")
+(common-lisp:export
+ '(*inspect-frontend* *inspect-browser* *inspect-print-lines*
+   *with-html-output-doctype* *user-mail-address*
+   *inspect-print-level* *inspect-print-length* *inspect-length*)
+ "CUSTOM")
+(ext:re-export "CUSTOM" "EXT")
+
+(common-lisp:in-package "EXT")
+(export
+ '(current-time with-http-output with-html-output)
+ "EXT")
+
 (in-package "SYSTEM")
 
-;(export '(inspect *inspect-frontend* *inspect-print-lines*
-;          *inspect-print-level* *inspect-print-length* *inspect-length*))
-
-(defmacro with-gensyms (syms &body body)
-  "Bind symbols to gensyms.  First sym is a string - `gensym' prefix.
-Inspired by Paul Graham, <On Lisp>, p. 145."
-  `(let (,@(mapcar (lambda (sy) `(,sy (gensym ,(car syms)))) (cdr syms)))
-    ,@body))
-
 (defun current-time (&optional (out t))
   "Print the current time to the stream (defaults to T)."
   (multiple-value-bind (se mi ho da mo ye) (get-decoded-time)
@@ -69,23 +73,30 @@ Inspired by Paul Graham, <On Lisp>, p. 145."
   (clos:with-slots (target-stream) stream (force-output target-stream)))
 (clos:defmethod stream-clear-output ((stream html-stream-out))
   (clos:with-slots (target-stream) stream (clear-output target-stream)))
+(clos:defmethod close ((stream html-stream-out) &rest opts)
+  (clos:with-slots (target-stream) stream (apply #'close target-stream opts))
+  (call-next-method))
+
+
+(defvar *with-html-output-doctype*
+  '("html" "PUBLIC" "\"-//W3C//DTD XHTML 1.0 Strict//EN\""
+    "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\""))
+(defvar *user-mail-address*
+  (concatenate 'string (or (getenv "USER") (getenv "USERNAME") "nobody") "@"
+               (let ((st (machine-instance)))
+                 (subseq st 0 (position #\Space st)))))
 
 (defmacro with-html-output ((var stream
-                             &key (doctype ''(html public
-                                              "-//W3C//DTD HTML 3.2//EN"))
+                             &key (doctype '*with-html-output-doctype*)
                                   (meta '(:http-equiv "Content-Type"
                                           :content "text/html"))
                                   base comment (title "untitled") (footer t)
                                   head)
                             &body body)
-  (with-gensyms ("HTML-" raw user mail mailto)
+  (with-gensyms ("HTML-" raw mailto)
     `(let* ((,raw ,stream)
             (,var (clos::make-instance 'html-stream-out :stream ,raw))
-            (,user (sys::getenv "USER"))
-            (,mail (concatenate 'string ,user "@"
-                                (let ((st (machine-instance)))
-                                  (subseq st 0 (position #\Space st)))))
-            (,mailto (concatenate 'string "mailto:" ,mail)))
+            (,mailto (concatenate 'string "mailto:" *user-mail-address*)))
       (macrolet ((with-tag ((tag &rest options) &body forms)
                    `(progn (format ,',raw "<~a~@{ ~a=~s~}>" ,tag ,@options)
                      ,@forms (format ,',raw "</~a>~%" ,tag)))
@@ -94,12 +105,12 @@ Inspired by Paul Graham, <On Lisp>, p. 145."
                      ,@forms (format ,',raw "</~a>" ,tag))))
         (unwind-protect
              (progn
-               (format ,raw "<!doctype~{ ~s~}>~%" ,doctype)
+               (format ,raw "<!DOCTYPE~{ ~a~}>~%" ,doctype)
                ;; print the comment
                (format ,raw "<!--~% Created on ") (current-time ,raw)
-               (format ,raw "~% by ~a@~a~% using `with-open-html'
+               (format ,raw "~% by ~a~% using `with-open-html'
  Lisp: ~a ~a~@[~%~a~]~% -->~2%"
-                       ,user (machine-instance)
+                       *user-mail-address*
                        (lisp-implementation-type) (lisp-implementation-version)
                        ,comment)
                (when ,base
@@ -107,7 +118,7 @@ Inspired by Paul Graham, <On Lisp>, p. 145."
                (with-tag (:html)
                  (with-tag (:head ,@head)
                    (with-tag (:meta ,@meta))
-                   (with-tag (:link :rev 'made :href ,mailto))
+                   (with-tag (:link :rev "made" :href ,mailto))
                    (with-tag (:title) (princ ,title ,var)))
                  (with-tag (:body)
                    ,@body
@@ -116,42 +127,49 @@ Inspired by Paul Graham, <On Lisp>, p. 145."
                        (with-tag (:hr))
                        (with-tag (:address)
                          (with-tag (:a :href ,mailto)
-                           (princ ,mail ,var)))
+                           (princ *user-mail-address* ,var)))
                        (with-tagl (:strong) (current-time ,var)))))))
+          (when ,var (close ,var))
           (when ,raw (close ,raw)))))))
 
-(defun crlf (sock)
-  "Write CR/LF into the socket SOCK."
-  (write-char (code-char 13) sock)
-  (write-char (code-char 10) sock))
+(defvar *http-encoding*
+  (make-encoding #+UNICODE :charset #+UNICODE charset:utf-8
+                 :line-terminator :dos))
 
 (defmacro with-http-output ((var raw &rest opts &key keep-alive (debug 0)
+                             (return-code 200) (return-name "OK")
                              &allow-other-keys)
                             &body body)
   "Write some HTML to an http client on socket stream RAW.
 Supplies some HTTP/1.0 headers and calls `with-html-output'."
-  (with-gensyms ("HTTP-" string stream sock header line)
-    (remf opts :keep-alive) (remf opts :debug)
+  (with-gensyms ("HTTP-" string vector stream sock header line dbg alive)
     `(let* ((,sock ,raw)
+            (,dbg ,debug) (,alive ,keep-alive)
             (,string (with-output-to-string (,stream)
-                       (with-html-output (,var ,stream ,@opts) ,@body)))
-            (,header (list "HTTP/1.0 200 OK" "Content-type: text/html"
-                           (format nil "Content-length: ~d" (length ,string))
-                           (format nil "Connection: ~:[Close~;Keep-Alive~]"
-                                   ,keep-alive))))
-      (dolist (,line ,header)
-        (write-string ,line ,sock)
-        (when (and ,debug (> ,debug 0))
-          (format t "<- ~a~%" ,line))
-        (crlf ,sock))
-      (crlf ,sock)
-      (write-string ,string ,sock)
-      (when (and ,debug (> ,debug 3))
-        (format t "<- ~s~%" ,string))
-      (unless ,keep-alive
-        (when (and ,debug (> ,debug 0))
-          (format t "~s: closing ~s~%" 'with-http-output ,sock))
-        (close ,sock)))))
+                       (with-html-output (,var ,stream ,@(remove-plist opts :keep-alive :debug :return-code :return-name))
+                         ,@body)))
+            (,vector (ext:convert-string-to-bytes ,string *http-encoding*))
+            (,header (list (format nil "HTTP/1.0 ~d ~a"
+                                   ,return-code ,return-name)
+                           #+UNICODE "Content-type: text/html; charset=utf-8"
+                           #-UNICODE "Content-type: text/html"
+                           (format nil "Content-length: ~d" (length ,vector))
+                           (format nil "Connection: ~:[close~;keep-alive~]"
+                                   ,alive))))
+       (dolist (,line ,header)
+         (write-line ,line ,sock)
+         (when (and ,dbg (> ,dbg 0))
+           (format t "<- ~a~%" ,line)))
+       (terpri ,sock)
+       (setf (stream-element-type ,sock) 'unsigned-byte)
+       (write-byte-sequence ,vector ,sock)
+       (setf (stream-element-type ,sock) 'character)
+       (when (and ,dbg (> ,dbg 3))
+         (format t "<- ~s~%" ,string))
+       (unless ,alive
+         (when (and ,dbg (> ,dbg 0))
+           (format t "~s: closing ~s~%" 'with-http-output ,sock))
+         (close ,sock)))))
 
 (defun flush-http (sock)
   "Read everything from the HTTP socket SOCK, until a blank line."
@@ -159,6 +177,15 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
         :while (and line (plusp (length line)))
         :collect line))
 
+(defun http-error (sock url &key (name "Not Found") (code 404)
+                   (keep-alive nil) (debug 0))
+  "Report a request error."
+  (with-http-output (out sock :keep-alive keep-alive :debug debug
+                          :return-code code :return-name name)
+    (with-tag (:h1) (princ name out))
+    (with-tag (:p)
+      (format out "The requested URL ~s was not found on this server." url))))
+
 ;;;
 ;;; options
 ;;;
@@ -170,8 +197,7 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
 (defvar *inspect-print-length* 10) ; default for `*print-length*'
 (defvar *inspect-length* 5)     ; the number of sequence elements to print
 
-;; all `inspection' objects in this session
-(defparameter *inspect-all* (make-array 10 :fill-pointer 0 :adjustable t))
+(defvar *inspect-all* nil) ; all `inspection' objects in this session
 (defparameter *inspect-debug* 0) ; debug level
 (defvar *inspect-unbound-value*) ; the value for the unbound slots
 
@@ -311,7 +337,10 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
              :num-slots (length slots)
              :nth-slot (lambda (ii)
                          (let ((slot (nth ii slots)))
-                           (values (clos:slot-value obj slot) slot)))
+                           (values (if (clos:slot-boundp obj slot)
+                                       (clos:slot-value obj slot)
+                                       *inspect-unbound-value*)
+                                   slot)))
              :set-slot (lambda (ii val)
                          (setf (clos:slot-value obj (nth ii slots)) val))
              :self obj :title "structure object"
@@ -377,6 +406,12 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
               (get-insp (insp-up insp) (1- (insp-pos insp)))))
         (:r (when (insp-pos insp)
               (get-insp (insp-up insp) (1+ (insp-pos insp)))))
+        (:w (setq *inspect-length* (* *inspect-length* 2))
+            (inspect-backend (insp-self insp) :up (insp-up insp)
+                             :pos (insp-pos insp) :id (insp-id insp)))
+        (:n (setq *inspect-length* (round *inspect-length* 2))
+            (inspect-backend (insp-self insp) :up (insp-up insp)
+                             :pos (insp-pos insp) :id (insp-id insp)))
         (t (when (and (integerp com) (< -1 com (insp-num-slots insp)))
              (inspect-backend (funcall (insp-nth-slot insp) com)
                               :up insp :pos com)))))))
@@ -395,10 +430,7 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
     (error "~s: unknown inspect front end: ~s" 'inspect-frontend frontend)))
 
 (clos:defgeneric inspect-finalize (frontend)
-  (:method ((frontend t))
-    (dotimes (ii (length *inspect-all*))
-      (setf (aref *inspect-all* ii) nil))
-    (setf (fill-pointer *inspect-all*) 0)))
+  (:method ((frontend t))))
 
 (defun inspect-read-clean-eval (insp stream)
   ;; `read' a form, destructively replace `:self' with INSP and `:slot'
@@ -429,7 +461,11 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
   (when (insp-nth-slot insp)
     (loop :for ii :from 0 :to (insp-num-slots-print insp)
           :do (multiple-value-bind (val name) (funcall (insp-nth-slot insp) ii)
-                (format out "~d~@[ [~a]~]:  ~s~%" ii name val)))))
+                (format out "~d~@[ [~a]~]:  ~s~%" ii name val)))
+    (let ((last (insp-last-slot insp)))
+      (when (< *inspect-length* last)
+        (format out "~:D more slot~:P available (use :e to increase ~S)"
+                (- last *inspect-length*) '*inspect-length*)))))
 
 (clos:defmethod inspect-frontend ((insp inspection) (frontend (eql :tty)))
   (print-inspection insp *terminal-io* frontend)
@@ -443,13 +479,16 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
       ((:h :?) (format t " *** commands:~% :h, :?~15t this help
  :p, :a~15t Print the current item Again
  :s~15t re-inspect this item (Self)
- :d~15t Describe the current item~%")
+ :d~15t Describe the current item
+ :w~15t Widen: double ~S and re-inspect
+ :n~15t Narrow: halve ~S and re-inspect
+" '*inspect-length* '*inspect-length*)
        (when (insp-up insp)
          (format t " :u~15t return UP to the parent~%"))
        (when (insp-left-p insp)
-         (format t " :l~15t inspect the left neighbor~%"))
+         (format t " :l~15t inspect the Left neighbor~%"))
        (when (insp-right-p insp)
-        (format t " :r~15t inspect the right neighbor~%"))
+        (format t " :r~15t inspect the Right neighbor~%"))
        (when (insp-nth-slot insp)
          (format t " number~15t inspect this slot~%"))
        (format t " :e lisp-form~15t eval this form, with these substitutions:
@@ -460,13 +499,15 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
        (force-output))
       (:d (describe (insp-self insp)))
       ((:p :a) (print-inspection insp *terminal-io* frontend))
-      (:e (handler-case (print (inspect-read-clean-eval insp *terminal-io*))
-            (error (err) (format t " *** error: ~s" err))))
+      (:e (handler-case (let ((v (inspect-read-clean-eval insp *terminal-io*)))
+                          (format t "~&~S~%" v))
+            (error (err) (format t " *** error: ~A" err))))
       (:m (handler-case
-              (print (funcall (insp-set-slot insp)
-                              (inspect-read-clean-eval insp *terminal-io*)
-                              (inspect-read-clean-eval insp *terminal-io*)))
-            (error (err) (format t " *** error: ~s" err))))
+              (let ((v (funcall (insp-set-slot insp)
+                                (inspect-read-clean-eval insp *terminal-io*)
+                                (inspect-read-clean-eval insp *terminal-io*))))
+                (format t "~&~S~%" v))
+            (error (err) (format t " *** error: ~A" err))))
       (t (cond ((setq insp (get-insp id com))
                 (print-inspection insp *terminal-io* frontend)
                 (setq id (insp-id insp)))
@@ -474,7 +515,7 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
                   (setq insp (get-insp id :s))))))))
 
 ;;;
-;;; HTTP backend
+;;; HTTP frontend
 ;;;
 
 (clos:defmethod print-inspection ((insp inspection) (raw stream)
@@ -496,7 +537,12 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
                       (with-tag (:li)
                         (with-tag (:a :href (href ii))
                           (princ (or name "inspect") out))
-                        (with-tag (:pre) (write val :stream out)))))))
+                        (with-tag (:pre) (write val :stream out))))))
+        (let ((last (insp-last-slot insp)))
+          (when (< *inspect-length* last)
+            (with-tag (:p)
+              (format out "~:D more slot~:P available"
+                      (- last *inspect-length*))))))
       (with-tag (:hr))
       (with-tag (:h2) (princ "describe:" out))
       (with-tag (:pre) (describe (insp-self insp) out))
@@ -514,95 +560,129 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
           (when (insp-up insp)
             (with-tag (:td :align "center")
               (with-tag (:a :href (href :u)) (princ "parent" out))))
+          (with-tag (:td :align "center")
+            (format out "~S: " '*inspect-length*)
+            (with-tag (:a :href (href :w)) (princ "double" out))
+            (princ " " out)
+            (with-tag (:a :href (href :n)) (princ "halve" out)))
           (with-tag (:td :align "right")
             (with-tag (:a :href (href :s)) (princ "self" out))))))))
 
 (defun http-command (server &key (debug *inspect-debug*) socket)
   "Accept a connection from the server, return the GET command and the socket."
   (when (> debug 1)
-    (format t "~s: server: ~s; socket: ~s ~%" 'http-command server socket))
-  (unless (and socket (open-stream-p socket))
-    (setq socket (socket-accept server))
-    (when (> debug 1)
-      (format t "~s: new socket: ~s~%" 'http-command socket)))
-  (let ((response (flush-http socket)) id com keep-alive)
-    (unless response (error "~s: no response" 'http-command))
+    (format t "~s: server: ~s; socket: ~s~%" 'http-command server socket))
+  (let (response id com keep-alive)
+    (loop (unless (and socket (open-stream-p socket))
+            (setq socket (socket-accept server
+                                        :external-format *http-encoding*))
+            (when (> debug 1)
+              (format t "~s: new socket: ~s~%" 'http-command socket)))
+          (setq response (flush-http socket))
+          (when response (return))
+          ;; connection timed out?
+          (close socket))
     (when (> debug 1)
       (dolist (line response)
         (format t "-> ~a~%" line)))
     (dolist (line response)
       (when (string-beg-with "Connection: " line)
-        (setq keep-alive (string= line "Keep-Alive" :start1 12))
+        (setq keep-alive (string-equal line "keep-alive" :start1 12))
         (when (> debug 0)
           (format t "~s: connection: ~s (keep-alive: ~s)~%"
-                  'http-command (subseq line 12) keep-alive)))
+                  'http-command (subseq line 12) keep-alive))
+        (when keep-alive
+          ;; we override `keep-alive' because it makes it impossible to
+          ;; switch browsers in the middle of an inspection session
+          (setq keep-alive nil)
+          (when (> debug 0)
+            (format t "~s: overriding keep-alive to NIL~%" 'http-command))))
       (when (string-beg-with "GET /" line)
         (let ((pos (position #\/ line :test #'char= :start 5)))
-          (setq id (parse-integer line :start 5 :end pos)
-                com (read-from-string line nil nil :start (1+ pos)))
-          (when (> debug 0)
-            (format t "~s: command: id=~d com=~s~%" 'http-command id com)))))
+          (setq id (parse-integer line :start 5 :end pos :junk-allowed t))
+          (cond (id
+                 (setq com
+                       (if (char= #\% (aref line (1+ pos))) ; %xx hex
+                           (progn ; "%3a" --> ":"
+                             (setf (aref line (+ 3 pos))
+                                   (code-char (parse-integer
+                                               line :radix 16 :start (+ 2 pos)
+                                               :end (+ 4 pos))))
+                             (read-from-string line nil nil :start (+ 3 pos)))
+                           (read-from-string line nil nil :start (1+ pos))))
+                 (when (> debug 0)
+                   (format t "~s: command: id=~d com=~s~%"
+                           'http-command id com)))
+                (t
+                 (http-error socket line :debug debug :keep-alive keep-alive)
+                 (when (> debug 0)
+                   (format t "~s: invalid request: ~s~%"
+                           'http-command line)))))))
     (values socket id com keep-alive)))
 
 (clos:defmethod inspect-frontend ((insp inspection) (frontend (eql :http)))
-  (do ((server
-        (let* ((server (socket-server)) (port (socket-server-port server))
-               (host (machine-instance)))
-          (when (> *inspect-debug* 0)
-            (format t "~&~s [~s]: server: ~s~%"
-                    'inspect-frontend frontend server))
-          (browse-url (format nil "http://~a:~d/0/:s"
-                              (if *inspect-browser* "127.0.0.1"
-                                  (subseq host 0 (position #\Space host)))
-                              port)
-                      :browser *inspect-browser*)
-          server))
-       sock id com keep-alive)
-      ((eq com :q) (socket-server-close server)
-       (when (open-stream-p sock)
-         (do () ((null (read-char-no-hang sock))))
-         (close sock)))
-    (setf (values sock id com keep-alive) (http-command server :socket sock))
-    (if (eq com :q)
-        (with-http-output (out sock :keep-alive keep-alive
-                               :debug *inspect-debug*
-                               :title "inspect" :footer nil)
-          (with-tag (:h1) (princ "thanks for using inspect" out))
-          (with-tag (:p) (princ "you may close this window now" out)))
-        (if (setq insp (get-insp id com))
-            (print-inspection insp sock frontend :keep-alive keep-alive)
-            (with-http-output (out sock :keep-alive keep-alive
-                                   :debug *inspect-debug*
-                                   :title "inspect" :footer nil)
-              (with-tag (:h1)
-                (format out "error: wrong command: ~:d/~s" id com))
-              (with-tag (:p)
-                (princ "either this is an old inspect session, or a " out)
-                (with-tag (:a :href "https://sourceforge.net/bugs/?func=addbug&group_id=1802") (princ "bug" out))))))
-    (when (> *inspect-debug* 0)
-      (format t "~s [~s]: cmd:~d/~s id:~d~%" 'inspect-frontend frontend
-              id com (insp-id insp)))))
-
+  (let ((server
+         (let* ((server (socket-server)) (port (socket-server-port server))
+                (host (machine-instance)))
+           (when (> *inspect-debug* 0)
+             (format t "~&~s [~s]: server: ~s~%"
+                     'inspect-frontend frontend server))
+           (browse-url (format nil "http://~a:~d/0/:s"
+                               (if *inspect-browser* "127.0.0.1"
+                                   (subseq host 0 (position #\Space host)))
+                               port)
+                       :browser *inspect-browser*)
+           server))
+        sock id com keep-alive)
+    (unwind-protect
+         (loop (when (eq com :q) (return))
+           (setf (values sock id com keep-alive)
+                 (http-command server :socket sock))
+           (when id
+             (if (eq com :q)
+               (with-http-output (out sock :keep-alive keep-alive
+                                      :debug *inspect-debug*
+                                      :title "inspect" :footer nil)
+                 (with-tag (:h1) (princ "thanks for using inspect" out))
+                 (with-tag (:p) (princ "you may close this window now" out)))
+               (if (setq insp (get-insp id com))
+                 (print-inspection insp sock frontend :keep-alive keep-alive)
+                 (with-http-output (out sock :keep-alive keep-alive
+                                        :debug *inspect-debug*
+                                        :title "inspect" :footer nil)
+                   (with-tag (:h1)
+                     (format out "error: wrong command: ~:d/~s" id com))
+                   (with-tag (:p)
+                     (princ "either this is an old inspect session, or a " out)
+                     (with-tag (:a :href "https://sourceforge.net/bugs/?func=addbug&group_id=1802") (princ "bug" out))))))
+             (when (> *inspect-debug* 0)
+               (format t "~s [~s]: cmd:~d/~s id:~d~%" 'inspect-frontend
+                       frontend id com (insp-id insp)))))
+      (socket-server-close server)
+      (when (open-stream-p sock)
+        (do () ((null (read-char-no-hang sock))))
+        (close sock)))))
 ;;;
 ;;; the juice
 ;;;
 
 ;;;###autoload
-(defun inspect (object &key (frontend *inspect-frontend*)
-                (browser *inspect-browser*))
+(defun inspect (object &key ((:frontend *inspect-frontend*) *inspect-frontend*)
+                ((:browser *inspect-browser*) *inspect-browser*))
   (let* ((*print-array* nil) (*print-pretty* t)
          (*print-circle* t) (*print-escape* t)
-         #-clisp (*print-lines* *inspect-print-lines*)
+         (*inspect-length* *inspect-length*)
+         (*print-lines* *inspect-print-lines*)
          (*print-level* *inspect-print-level*)
          (*print-length* *inspect-print-length*)
-         (*package* (make-package (gensym "INSPECT-TMP-PACKAGE-")))
-         (*inspect-unbound-value* (intern "#<unbound>" *package*))
-         (*inspect-frontend* frontend)
-         (*inspect-browser* browser))
+         (*inspect-all* (make-array 10 :fill-pointer 0 :adjustable t))
+         (tmp-pack (make-package (gensym "INSPECT-TMP-PACKAGE-")))
+         (*package* tmp-pack)
+         (*inspect-unbound-value* (intern "#<unbound>" tmp-pack)))
     (unwind-protect
-         (inspect-frontend (inspect-backend object) frontend)
-      (inspect-finalize frontend)
-      (delete-package *package*))
+         (inspect-frontend (inspect-backend object) *inspect-frontend*)
+      (inspect-finalize *inspect-frontend*)
+      (delete-package tmp-pack))
     (values)))
 
 ;;; inspect.lisp ends here
