@@ -1,5 +1,5 @@
 ;;; MAKE-LOAD-FORM for CLISP
-;;; Sam Steingold 2001-2004
+;;; Sam Steingold 2001-2004, 2009
 ;;; Bruno Haible 2004
 
 ;; this could have been placed in in clos.lisp,
@@ -14,10 +14,10 @@
         (etypecase object
           (standard-object
            (mapcan (lambda (slot)
-                     (when (eq :instance (slotdef-allocation slot))
-                       (list (slotdef-name slot))))
+                     (when (eq (slot-definition-allocation slot) ':instance)
+                       (list (slot-definition-name slot))))
                    slots))
-          (structure-object (mapcar #'slotdef-name slots))))))
+          (structure-object (mapcar #'slot-definition-name slots))))))
   (declare (ignore environment))
   (values `(allocate-instance (find-class ',(class-name (class-of object))))
           `(progn
@@ -55,34 +55,45 @@
   (:method ((object condition) &optional environment)
     (declare (ignore environment))
     (signal-missing-load-form object))
-  (:method ((object class) &optional environment)
+  (:method ((object defined-class) &optional environment)
     (declare (ignore environment))
     ;; TODO: Implement as described in CLHS
     `(find-class ',(class-name object))))
 
 (defun mlf-init-function (object)
-  (multiple-value-bind (cre-form ini-form) (make-load-form object)
-    (if ini-form
-        (let ((var (gensym "MAKE-LOAD-FORM-")))
-          `(lambda ()
-            (let ((,var ,cre-form))
-              ,(sublis `((',object . ,var) (,object . ,var)) ini-form
-                       :test #'equal)
-              ,var)))
-        `(lambda () ,cre-form))))
+  (multiple-value-bind (creation-form initialization-form)
+      (make-load-form object)
+    (let ((funname
+            (gensym
+              (sys::string-concat
+                "CREATE-INSTANCE-OF-<"
+                (write-to-string (class-name (class-of object)) :readably nil)
+                ">-"))))
+      `(FUNCTION ,funname
+         (LAMBDA ()
+           ,@(if (and sys::*compiling* sys::*compiling-from-file*)
+               '((DECLARE (COMPILE)))
+               '())
+           ,(if initialization-form
+              (let ((var (gensym "MAKE-LOAD-FORM-")))
+                `(LET ((,var ,creation-form))
+                   ,(sublis `((',object . ,var) (,object . ,var))
+                            initialization-form
+                            :test #'equal)
+                   ,var))
+              creation-form))))))
 
 (defun make-init-form (object)
-  (when compiler::*load-forms*
+  (when sys::*load-forms*
     (multiple-value-bind (form found-p)
-        (gethash object compiler::*load-forms*)
+        (gethash object sys::*load-forms*)
       (if found-p
         form
-        (setf (gethash object compiler::*load-forms*)
+        (setf (gethash object sys::*load-forms*)
               (block compute-init-form
                 (handler-bind
                   ((missing-load-form
                      #'(lambda (err)
                          (when (eql (missing-load-form-object err) object)
                            (return-from compute-init-form nil)))))
-                  ; TODO: Explain why it's worth invoking the compiler here.
-                  `(funcall ,(compile nil (mlf-init-function object))))))))))
+                  `(funcall ,(eval (mlf-init-function object))))))))))
