@@ -47,7 +47,8 @@
 #include <stdio.h>
 
 /* Boolean type.  */
-typedef int boolean_t;
+/* Not a typedef because AIX <sys/types.h> already defines boolean_t.  */
+#define boolean_t int
 #define FALSE 0
 #define TRUE 1
 
@@ -693,7 +694,7 @@ new_statement (void)
     {
       /* Start a new function.  */
       func_number++; func_statements = 0;
-      printf ("void main%d (TEXT *curr)\n", func_number);
+      printf ("static void main%d (TEXT *curr)\n", func_number);
       printf ("{\n");
       {
         string_t main_line;
@@ -1427,13 +1428,13 @@ main (int argc, char *argv[])
     for (i = 0; i < main_output.index; i++)
       emit_control_directive (&main_output.data[i]);
   }
-  printf ("  if (fflush (stdout) || ferror (stdout))\n");
+  printf ("  if (ferror (stdout) || fclose (stdout))\n");
   printf ("    exit (1);\n");
   printf ("  exit (0);\n");
   printf ("}\n");
 
   /* Done. */
-  if (fflush (stdout) || ferror (stdout))
+  if (ferror (stdout) || fclose (stdout))
     exit (1);
   exit (0);
 }
